% Scale a vector of logs (usually unscaled log probabilities) so that the sum is 1 in
% the non-log space. 
%
% This routine uses the trick of KJB library of avoiding very small numbers
% during rescaling due to exponentiating large negative log numbers. It
% first subtracts the maximum from all the values in the log vector and then
% exponentiates them followed by rescaling. This ensures that at least one log
% value is zero before the exponent is taken. It returns the log of the sum of
% exponentiated values, which usually amounts to the log likelihood in a mixture
% model.

function [scaled_vec, log_lhood] = scale_by_sum_log_vec(log_vec)

max_log           = max(log_vec);
off_log_vec       = log_vec - max_log;
unlog_off_log_vec = exp(off_log_vec);
sum_unlogs        = sum(unlog_off_log_vec);
scaled_vec        = unlog_off_log_vec ./ sum_unlogs;
log_lhood         = log(sum_unlogs) + max_log;
