% Compute average accuracy of label prediction on held-out samples of Whitfield et al.
% data. 
%
% A number of models for joint clustering of gene expression levels and GO tags
% are learnt using EM with different random initializations. Accuracy of label
% prediction is computed for each model and the accuracies are averaged. Average
% accuracy of label prediction by clustering only gene expression levels is also
% computed.

function [] = test_predict_max_lhood_label_state_specific_dependent_whitfield(n_tries, ...
                                                                              acc_results_file, ...
                                                                              go_dag, ...
                                                                              state_switch_pts, ...
                                                                              n_clusters, ...
                                                                              stage_labels_file)

addpath(genpath('..'));

n_em_iters              = 40;
do_weight_GO_tags       = 0;
do_marginalize          = 1;
ignore_GO_tags_for_pred = 0;
do_state_pred_from_time = 0;
do_force_state_switches = 1;
forced_state_switch_pts_cy_1 = [state_switch_pts(1)];
forced_state_switch_pts_cy_2 = [state_switch_pts(2)];                
forced_state_switch_pts_cy_3 = [state_switch_pts(3)];                
do_gene_identifiability = 1;
do_count_GO_tags_once   = 1;
do_share_clusters       = 0;
do_arithmetic_avg       = 1;

n_states                 = 2;
num_clusters             = n_clusters;
n_clusters               = num_clusters*ones(1, n_states);

results_dir = 'models/whitfield_dataset/em_clustering_with_GO_tags_state_specific_dependent/pred-w-GO-tags-wo-weights';

train_files = {'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt', ...
               'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt'};

test_files = {'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_3.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_1.txt', ...
              'data/whitfield_dataset/gene_expr_tt3_1099_cy_2.txt'};

ng_go_tags_files = {'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099', ...
                    'data/whitfield_dataset/no_go_tags_1099'};

bp_go_tags_files = {'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_bp_w_anc_1099'};
                    
mf_go_tags_files = {'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099', ...
                    'data/whitfield_dataset/go_tags_mf_w_anc_1099'};
                    
ng_clusters_files = {strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_ng_1099_cy_3')};

bp_clusters_files = {strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_bp_1099_cy_3')};
                 
mf_clusters_files = {strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_1'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_2'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_3'), ...
                     strcat(results_dir, '/', 'bayesian_go_input_mf_1099_cy_3')};

ng_p_o_bar_c_files ={strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_ng_1099_cy_3')};

bp_p_o_bar_c_files ={strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_bp_w_anc_1099_cy_3')};

mf_p_o_bar_c_files ={strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_1'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_2'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_3'), ...
                     strcat(results_dir, '/', 'p_o_bar_c_mf_w_anc_1099_cy_3')};

ng_model_files = {strcat(results_dir, '/', 'model_ng_1099_cy_1.mat'), ... 
                  strcat(results_dir, '/', 'model_ng_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_ng_1099_cy_3.mat')};

bp_model_files = {strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_1.mat'), ... 
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_bp_w_anc_1099_cy_3.mat')};

mf_model_files = {strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_1.mat'), ... 
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_1.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_2.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_3.mat'), ...
                  strcat(results_dir, '/', 'model_mf_w_anc_1099_cy_3.mat')};

n_exps = length(train_files);

acc_s_ng = zeros(n_exps*n_tries, 1);
acc_s_bp = zeros(n_exps*n_tries, 1);
acc_s_mf = zeros(n_exps*n_tries, 1);

% Making script restartable.
if (exist(acc_results_file , 'file'))
    
    acc_from_file  = load(acc_results_file);
    sum_pred_probs = sum(acc_from_file(:));    

    if (sum_pred_probs > 0)
        acc_s_ng = acc_from_file(:, 1);
        acc_s_bp = acc_from_file(:, 2);
        acc_s_mf = acc_from_file(:, 3);
    end

else
    
    acc_from_file = [acc_s_ng acc_s_bp acc_s_mf];

end

% Writing stage prediction labels.
max_n_samples     = 17;
stage_pred_labels = zeros(n_exps*n_tries, 3*max_n_samples);

for i_exp = 1:n_exps
    train_file = train_files{i_exp};
    test_file  = test_files{i_exp};

    if ( strcmp(go_dag, 'ng') ) 
        
        % No GO tags.
        go_tags_file   = ng_go_tags_files{i_exp};
        clusters_file  = ng_clusters_files{i_exp};
        p_o_bar_c_file = ng_p_o_bar_c_files{i_exp};
        model_mat_file = ng_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                (n_exps*n_tries), 1));
            
            if ( sum_pred_probs  > 0 )
            
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else
                
                if (~isempty(strfind(train_file, 'cy_1')))
                    forced_state_switch_pts = forced_state_switch_pts_cy_1;
                elseif (~isempty(strfind(train_file, 'cy_2')))
                    forced_state_switch_pts = forced_state_switch_pts_cy_2;
                else
                    forced_state_switch_pts = forced_state_switch_pts_cy_3;
                end

                cluster_with_em_GO_tags_state_specific_dependent(train_file, ...
                                                                 go_tags_file, ...
                                                                 n_states, ...
                                                                 n_clusters, ...
                                                                 n_em_iters, ...
                                                                 do_weight_GO_tags, ...
                                                                 do_count_GO_tags_once, ...
                                                                 do_share_clusters, ...
                                                                 do_arithmetic_avg, ...
                                                                 model_mat_file, ...
                                                                 forced_state_switch_pts);

                [time_pred_probs, state_pred_probs] = predict_max_lhood_label_state_specific_dependent(train_file, ...
                                                                                                       test_file, ...
                                                                                                       go_tags_file, ...
                                                                                                       model_mat_file, ...
                                                                                                       do_marginalize, ...
                                                                                                       do_weight_GO_tags, ...
                                                                                                       ignore_GO_tags_for_pred, ...
                                                                                                       do_count_GO_tags_once, ...
                                                                                                       do_gene_identifiability, ...
                                                                                                       do_arithmetic_avg);

                n_samples   = size(time_pred_probs, 1);
                n_correct_s = 0;

                for s = 1:n_samples
                    
                    if (do_state_pred_from_time)
                        if (~isempty(strfind(train_file, 'cy_1')))

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_1(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(1):14))];

                        elseif (~isempty(strfind(train_file, 'cy_2')))

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_2(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(1):14))];

                        else

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_3(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_3(1):17))];
                            
                        end
                    end
                        
                    [max_prob, pred_stage] = max(state_pred_probs(s, :));

                    stage_pred_labels(((i_exp-1)*n_tries)+trial, s) = pred_stage;

                    if (~isempty(strfind(test_file, 'cy_1')))

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_1(1)) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_1(1)) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    elseif (~isempty(strfind(test_file, 'cy_2')))

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_2(1)) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_2(1)) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end
                    
                    else

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_3(1)) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_3(1)) & (s <= 17) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    end
                end
                acc_s_ng(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;        
                
                %Update results.
                acc_results = [acc_s_ng acc_s_bp acc_s_mf];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(stage_labels_file, 'stage_pred_labels', '-ASCII');
            end

        end

    end

    if ( strcmp(go_dag, 'bp') )
        
        % BP GO tags.
        go_tags_file   = bp_go_tags_files{i_exp};
        clusters_file  = bp_clusters_files{i_exp};
        p_o_bar_c_file = bp_p_o_bar_c_files{i_exp};
        model_mat_file = bp_model_files{i_exp};

        for trial = 1:n_tries
            
            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                (n_exps*n_tries), 2));
            
            if ( sum_pred_probs  > 0 )
            
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else
            
                if (~isempty(strfind(train_file, 'cy_1')))
                    forced_state_switch_pts = forced_state_switch_pts_cy_1;
                elseif (~isempty(strfind(train_file, 'cy_2')))
                    forced_state_switch_pts = forced_state_switch_pts_cy_2;
                else
                    forced_state_switch_pts = forced_state_switch_pts_cy_3;
                end

                cluster_with_em_GO_tags_state_specific_dependent(train_file, ...
                                                                 go_tags_file, ...
                                                                 n_states, ...
                                                                 n_clusters, ...
                                                                 n_em_iters, ...
                                                                 do_weight_GO_tags, ...
                                                                 do_count_GO_tags_once, ...
                                                                 do_share_clusters, ...
                                                                 do_arithmetic_avg, ...
                                                                 model_mat_file, ...
                                                                 forced_state_switch_pts);

                [time_pred_probs, state_pred_probs] = predict_max_lhood_label_state_specific_dependent(train_file, ...
                                                                                                       test_file, ...
                                                                                                       go_tags_file, ...
                                                                                                       model_mat_file, ...
                                                                                                       do_marginalize, ...
                                                                                                       do_weight_GO_tags, ...
                                                                                                       ignore_GO_tags_for_pred, ...
                                                                                                       do_count_GO_tags_once, ...
                                                                                                       do_gene_identifiability, ...
                                                                                                       do_arithmetic_avg);
 
                n_samples   = size(time_pred_probs, 1);
                n_correct_s = 0;

                for s = 1:n_samples
                    
                    if (do_state_pred_from_time)
                        if (~isempty(strfind(train_file, 'cy_1')))

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_1(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(1):14))];

                        elseif (~isempty(strfind(train_file, 'cy_2')))

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_2(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(1):14))];

                        else

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_3(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_3(1):17))];
                            
                        end
                    end
                        
                    [max_prob, pred_stage] = max(state_pred_probs(s, :));

                    stage_pred_labels(((i_exp-1)*n_tries)+trial, max_n_samples+s) = pred_stage;

                    if (~isempty(strfind(test_file, 'cy_1')))

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_1(1)) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_1(1)) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    elseif (~isempty(strfind(test_file, 'cy_2')))

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_2(1)) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_2(1)) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end
                    
                    else

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_3(1)) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_3(1)) & (s <= 17) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    end
                end
                acc_s_bp(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;

                %Update results.
                acc_results = [acc_s_ng acc_s_bp acc_s_mf];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(stage_labels_file, 'stage_pred_labels', '-ASCII');
            end

        end

    end

    if ( strcmp(go_dag, 'mf') )

        % MF GO tags.
        go_tags_file   = mf_go_tags_files{i_exp};
        clusters_file  = mf_clusters_files{i_exp};
        p_o_bar_c_file = mf_p_o_bar_c_files{i_exp};
        model_mat_file = mf_model_files{i_exp};

        for trial = 1:n_tries

            sum_pred_probs = sum(acc_from_file((((i_exp-1)*n_tries)+trial): ...
                                                (n_exps*n_tries), 3));
            
            if ( sum_pred_probs  > 0 )
            
                exp_str   = num2str(i_exp);
                trial_str = num2str(trial);

                disp(strcat('Skipping trial: ', trial_str, ' for experiment: ', exp_str));
                continue;

            else
            
                if (~isempty(strfind(train_file, 'cy_1')))
                    forced_state_switch_pts = forced_state_switch_pts_cy_1;
                elseif (~isempty(strfind(train_file, 'cy_2')))
                    forced_state_switch_pts = forced_state_switch_pts_cy_2;
                else
                    forced_state_switch_pts = forced_state_switch_pts_cy_3;
                end

                cluster_with_em_GO_tags_state_specific_dependent(train_file, ...
                                                                 go_tags_file, ...
                                                                 n_states, ...
                                                                 n_clusters, ...
                                                                 n_em_iters, ...
                                                                 do_weight_GO_tags, ...
                                                                 do_count_GO_tags_once, ...
                                                                 do_share_clusters, ...
                                                                 do_arithmetic_avg, ...
                                                                 model_mat_file, ...
                                                                 forced_state_switch_pts);

                [time_pred_probs, state_pred_probs] = predict_max_lhood_label_state_specific_dependent(train_file, ...
                                                                                                       test_file, ...
                                                                                                       go_tags_file, ...
                                                                                                       model_mat_file, ...
                                                                                                       do_marginalize, ...
                                                                                                       do_weight_GO_tags, ...
                                                                                                       ignore_GO_tags_for_pred, ...
                                                                                                       do_count_GO_tags_once, ...
                                                                                                       do_gene_identifiability, ...
                                                                                                       do_arithmetic_avg);
                
                n_samples   = size(time_pred_probs, 1);
                n_correct_s = 0;

                for s = 1:n_samples
                    
                    if (do_state_pred_from_time)
                        if (~isempty(strfind(train_file, 'cy_1')))

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_1(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_1(1):14))];

                        elseif (~isempty(strfind(train_file, 'cy_2')))

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_2(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_2(1):14))];

                        else

                            state_pred_probs(s, :) = [sum(time_pred_probs(s, 1:(forced_state_switch_pts_cy_3(1)-1))), ...
                                                      sum(time_pred_probs(s, forced_state_switch_pts_cy_3(1):17))];
                            
                        end
                    end
                        
                    [max_prob, pred_stage] = max(state_pred_probs(s, :));

                    stage_pred_labels(((i_exp-1)*n_tries)+trial, (2*max_n_samples)+s) = pred_stage;

                    if (~isempty(strfind(test_file, 'cy_1')))

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_1(1)) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_1(1)) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    elseif (~isempty(strfind(test_file, 'cy_2')))

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_2(1)) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_2(1)) & (s <= 14) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end
                    
                    else

                        if ( (s >= 1) & (s < forced_state_switch_pts_cy_3(1)) )                
                            if (pred_stage == 1)
                                n_correct_s = n_correct_s + 1;
                            end
                        else %( (s >= forced_state_switch_pts_cy_3(1)) & (s <= 17) )
                            if (pred_stage == 2)
                                n_correct_s = n_correct_s + 1;
                            end
                        end

                    end
                end
                acc_s_mf(((i_exp-1)*n_tries)+trial) = n_correct_s / n_samples;
                
                %Update results.
                acc_results = [acc_s_ng acc_s_bp acc_s_mf];    
                save(acc_results_file, 'acc_results', '-ASCII');

                save(stage_labels_file, 'stage_pred_labels', '-ASCII');
            end

        end

    end

end
